import Foundation

import Model
import CustomTypes

let ai = RandomPlayer(name: "Le robot a mythos", piece_type: .A)
let cli = HumanPlayer(name: "Humain", piece_type: .B, callback: {
    allowed_moves, board in
    print("Available moves:")
    for i in allowed_moves.indices {
        print("\(i + 1)\t\(allowed_moves[i])")
    }
    print(">")
    
    guard let input = readLine(strippingNewline: true) else {
        fatalError("Invalid input")
    }
    
    guard let id = Int(input), id >= 1, id <= allowed_moves.count else {
        fatalError("Invalid move id: \(input)")
    }
    
    return allowed_moves[id - 1]
})

let players = [ cli, ai ]

guard var rules = FourInARowRules(
    columns: FourInARowRules.COLUMNS_DEFAULT,
    rows: FourInARowRules.ROWS_DEFAULT,
    players: players
) else {
    fatalError("Rules settings are invalid")
}

var board = rules.createBoard()

guard rules.isValid(board: board) else {
    fatalError("Board is invalid")
}

var state = rules.gameState(board: board, last_turn: nil)
print(state)

print("All moves:")
for move in rules.validMoves(board: board) {
    print(move)
}

print("Moves for \(PieceType.A):")
for move in rules.validMoves(board: board, for_player: ai) {
    print(move)
}

print(board)

while case .Playing(let turn) = state {
    let action = turn.chooseMove(
        allowed_moves: rules.validMoves(board: board, for_player: turn),
        board: board
    )
    
    let move = Move(player: turn, action: action)
    guard rules.isValid(board: board, move: move) else {
        fatalError("Move of \(turn) is invalid!")
    }
    
    switch move.action {
    case .InsertAt(let coords):
        board[coords] = Piece(type: move.player.piece_type)
    case .InsertOnSide(let side, let offset):
        switch board.fallCoordinates(
            initialCoords: board.getInsertionCoordinates(from: side, offset: offset),
            direction: side.opposite
        ) {
        case .Border(let at), .Piece(let at, _):
            board[at] = Piece(type: move.player.piece_type)
        case .Occupied:
            fatalError("Not supported")
        }
    }
    
    print(board)
    
    state = rules.gameState(board: board, last_turn: turn)
}

if case .Win(let winner, _) = state {
    print("\(winner.name) is the winner")
} else {
    print("Draw.")
}

exit(EXIT_SUCCESS)
